/*
 * Copyright (c) 2022 Libre Solar Technologies GmbH
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/drivers/uart.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/pm/device.h>
#include <algorithm>
#include <string.h>

#define LOG_LEVEL CONFIG_LOG_DEFAULT_LEVEL
#include <zephyr/logging/log.h>
LOG_MODULE_REGISTER(main_app);

#define UART_DEVICE_NODE DT_CHOSEN(zephyr_shell_uart)
static const struct device *const uart_dev = DEVICE_DT_GET(UART_DEVICE_NODE);

/*
 * Read characters from UART until line end is detected. Afterwards push the
 * data to the message queue.
 */
void serial_cb(const struct device *dev, void *user_data)
{
    uint8_t c;

    if (!uart_irq_update(dev))
    {
        return;
    }

    // if (uart_irq_tx_ready(dev))
    //{
    // }

    if (!uart_irq_rx_ready(dev))
    {
        return;
    }

    /* read until FIFO empty */
    while (uart_fifo_read(dev, &c, 1) == 1)
    {
    }
}

int uart_tx(uint8_t *buf, uint8_t msg_len)
{
    // uart_irq_tx_enable(uart_dev);
    for (uint8_t i = 0; i < msg_len; i++)
    {
        (uart_dev, buf[i]);
    }
    // uart_irq_tx_disable(uart_dev);
    return 0;
}

void enable()
{
    int err = pm_device_action_run(uart_dev, PM_DEVICE_ACTION_RESUME);
    if (err)
    {
        LOG_ERR("pm_device_action_run() failed (%d)\n", err);
    }
}

void disable()
{
    int err = pm_device_action_run(uart_dev, PM_DEVICE_ACTION_SUSPEND);
    if (err)
    {
        LOG_ERR("pm_device_action_run() failed (%d)\n", err);
    }
}

int init_once()
{
    if (!device_is_ready(uart_dev))
    {
        LOG_ERR("UART device not found!");
        return 0;
    }

    /* configure interrupt and callback to receive data */
    int ret = uart_irq_callback_user_data_set(uart_dev, serial_cb, NULL);
    if (ret < 0)
    {
        if (ret == -ENOTSUP)
        {
            LOG_ERR("Interrupt-driven UART API support not enabled\n");
        }
        else if (ret == -ENOSYS)
        {
            LOG_ERR("UART device does not support interrupt-driven API\n");
        }
        else
        {
            LOG_ERR("Error setting UART callback: %d\n", ret);
        }
        return 0;
    }
    uart_irq_rx_enable(uart_dev);
    //  uart_irq_tx_enable(uart_dev);

    disable();
    return 0;
}

int main(void)
{
    init_once();
    while (1)
    {
        enable();
        k_sleep(K_MSEC(10));
        LOG_INF("TX");
        uint8_t data[] = {0x10, 0x5b, 0xfe, 0x59, 0x16};
        uart_tx(data, sizeof(data));

        disable();

        k_sleep(K_SECONDS(2));
    }

    return 0;
}
